/* LICENSE BLOCK
Part of mde: a Modular D game-oriented Engine
Copyright © 2007-2008 Diggory Hardy

This program is free software: you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation, either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. */

/** Contains a basic colour type. */
module mde.types.Colour;

/// Represent a colour using clamped floats
struct Colour {
    /// Returns GL_ONE if total value is nearer white than black, else GL_ZERO.
    uint nearestGLConst () {
        return r+g+b >= 1.5f ? 1u : 0u;
    }
    
    float r,g,b;	/// values
    
    static {
        /// Predefined colours
        const Colour WHITE = { r:1f, g:1f, b:1f };
        const Colour BLACK = { r:0f, g:0f, b:0f };	/// ditto
        
        /// Construct from floats (doesn't clamp, but GL does when values are passed)
        Colour opCall (float r, float g, float b) {
            Colour c;
            c.r = r;
            c.g = g;
            c.b = b;
            return c;
        }
        /// Construct from ubytes
        Colour opCall (ubyte r, ubyte g, ubyte b) {
            Colour c;
            c.r = cast(float) r / 255f;
            c.g = cast(float) g / 255f;
            c.b = cast(float) b / 255f;
            return c;
        }
        /// Construct from 0xRRGGBB
        Colour opCall (int col) {
            Colour c;
            c.r = cast(float) ((col >> 16u) & 0xFF) / 255f;
            c.g = cast(float) ((col >> 8u)  & 0xFF) / 255f;
            c.b = cast(float) (col          & 0xFF) / 255f;
            return c;
        }
    }
}
